import logging
import json
from zeebe.base import ZeebeBase
from zeebe_grpc import gateway_pb2
from zeebe.exceptions import ZeebeWorkerException


class ZeebeWorker(ZeebeBase):
    """Represents a zeebe Worker"""

    def __init__(self, zeebe_address: str = None, client_id: str = None, client_secret: str = None,
                 name: str = "Zeebe Worker", timeout=1000, max_jobs=1, pollingtime=5):
        super().__init__(zeebe_address, client_id, client_secret)
        self.name = name
        self.timeout = timeout  # 1000 ms = 1 s
        self.max_jobs = max_jobs
        self.pollingtime = pollingtime  # 5s
        self.tasks = dict()

    def register_task(self, task_type, function, custom_timeout=None):
        self.tasks[task_type] = {
            "function": function,
            "timeout": custom_timeout or self.timeout,
        }

    def run(self):
        while True:
            logging.info("Polling Jobs...")

            for task_type, task in self.tasks.items():
                function = task.get("function")
                timeout = task.get("timeout")

                logging.info(f"Activating Job Type {task_type}")

                try:
                    activate_jobs_response = self.stub.ActivateJobs(
                        gateway_pb2.ActivateJobsRequest(
                            type=task_type,
                            timeout=timeout,
                            worker=self.name,
                            maxJobsToActivate=self.max_jobs,
                        )
                    )
                except Exception as e:
                    logging.error(e)
                    continue

                for response in activate_jobs_response:
                    for job in response.jobs:
                        try:
                            result = function(job)
                            self.stub.CompleteJob(
                                gateway_pb2.CompleteJobRequest(
                                    jobKey=job.key,
                                    variables=result or json.dumps({})
                                )
                            )
                            logging.info(f"Worker {self.name}: Task {task} completed")
                        except Exception as e:
                            while True:
                                try:
                                    self.stub.FailJob(
                                        gateway_pb2.FailJobRequest(
                                            jobKey=job.key,
                                            errorMessage=f"Worker {self.name}: Task {task_type} failed. Error: {e}"
                                        )
                                    )
                                except Exception as e:
                                    logging.error(e)
                                    continue

                                break

                            logging.error(f"Worker {self.name}: Task {task} failed")
                            raise ZeebeWorkerException(e)
